/*
 *  Copyright 2012 Alexey Andreev.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.teavm.javascript.ast;

import java.util.Map;
import org.teavm.model.ValueType;

/**
 *
 * @author Alexey Andreev
 */
public class NewArrayExpr extends Expr {
    private ValueType type;
    private Expr length;

    public ValueType getType() {
        return type;
    }

    public void setType(ValueType type) {
        this.type = type;
    }

    public Expr getLength() {
        return length;
    }

    public void setLength(Expr length) {
        this.length = length;
    }

    @Override
    public void acceptVisitor(ExprVisitor visitor) {
        visitor.visit(this);
    }

    @Override
    protected Expr clone(Map<Expr, Expr> cache) {
        Expr known = cache.get(this);
        if (known != null) {
            return known;
        }
        NewArrayExpr copy = new NewArrayExpr();
        cache.put(this, copy);
        copy.setType(type);
        copy.setLength(length != null ? length.clone() : null);
        return copy;
    }
}
